@@ -1,7 +1,7 @@
 /* grpatch.c -- main entry point for execution of patching routines.
  *
  *
- * SCL; 2012, 2013.
+ * SCL; 2012-2014.
  */
 
 
@@ -30,7 +30,7 @@ extern ptree_t *evar_list;
 extern ptree_t *svar_list;
 extern ptree_t *env_init;
 extern ptree_t *sys_init;
-ptree_t *env_trans = NULL;  /* Built from component parse trees in env_trans_array. */
+ptree_t *env_trans = NULL;  /* Built from components in env_trans_array. */
 ptree_t *sys_trans = NULL;
 extern ptree_t **env_goals;
 extern ptree_t **sys_goals;
@@ -52,7 +52,7 @@ extern ptree_t *gen_tree_ptr;
 #define OUTPUT_FORMAT_TULIP 1
 #define OUTPUT_FORMAT_DOT 2
 #define OUTPUT_FORMAT_AUT 3
-#define OUTPUT_FORMAT_TULIP0 4
+#define OUTPUT_FORMAT_JSON 5
 
 /* Runtime modes */
 #define GR1C_MODE_UNSET 0
@@ -66,7 +66,7 @@ int main( int argc, char **argv )
 	bool help_flag = False;
 	bool ptdump_flag = False;
 	bool logging_flag = False;
-	byte format_option = OUTPUT_FORMAT_TULIP;
+	byte format_option = OUTPUT_FORMAT_JSON;
 	unsigned char verbose = 0;
 	int input_index = -1;
 	int edges_input_index = -1;  /* If patching, command-line flag "-e". */
@@ -79,8 +79,14 @@ int main( int argc, char **argv )
 	int clformula_index = -1;  /* For command-line flag "-f". */
 	ptree_t *clformula = NULL;
 
+	/* System goal to be removed; -1 indicates unset.  Order matches
+	   that of the given specification file, which should also be
+	   consistent with goal mode labeling of the given automaton. */
+	int remove_goal_mode = -1;  /* For command-line flag "-r". */
+
 	int i, j, var_index;
 	ptree_t *tmppt;  /* General purpose temporary ptree pointer */
+	char *tmpstr = NULL;
 
 	DdManager *manager;
 	DdNode *T = NULL;
@@ -97,7 +103,8 @@ int main( int argc, char **argv )
 			if (argv[i][1] == 'h') {
 				help_flag = True;
 			} else if (argv[i][1] == 'V') {
-				printf( "grpatch (experiment-related program, distributed with gr1c v" GR1C_VERSION ")\n\n" GR1C_COPYRIGHT "\n" );
+				printf( "grpatch (experiment-related program, distributed with"
+						" gr1c v" GR1C_VERSION ")\n\n" GR1C_COPYRIGHT "\n" );
 				return 0;
 			} else if (argv[i][1] == 'v') {
 				verbose = 1;
@@ -125,16 +132,17 @@ int main( int argc, char **argv )
 				}
 				if (!strncmp( argv[i+1], "txt", strlen( "txt" ) )) {
 					format_option = OUTPUT_FORMAT_TEXT;
-				} else if (!strncmp( argv[i+1], "tulip0", strlen( "tulip0" ) )) {
-					format_option = OUTPUT_FORMAT_TULIP0;
 				} else if (!strncmp( argv[i+1], "tulip", strlen( "tulip" ) )) {
 					format_option = OUTPUT_FORMAT_TULIP;
 				} else if (!strncmp( argv[i+1], "dot", strlen( "dot" ) )) {
 					format_option = OUTPUT_FORMAT_DOT;
 				} else if (!strncmp( argv[i+1], "aut", strlen( "aut" ) )) {
 					format_option = OUTPUT_FORMAT_AUT;
+				} else if (!strncmp( argv[i+1], "json", strlen( "json" ) )) {
+					format_option = OUTPUT_FORMAT_JSON;
 				} else {
-					fprintf( stderr, "Unrecognized output format. Try \"-h\".\n" );
+					fprintf( stderr,
+							 "Unrecognized output format. Try \"-h\".\n" );
 					return 1;
 				}
 				i++;
@@ -168,6 +176,19 @@ int main( int argc, char **argv )
 				run_option = GR1C_MODE_PATCH;
 				clformula_index = i+1;
 				i++;
+			} else if (argv[i][1] == 'r') {
+				if (i == argc-1) {
+					fprintf( stderr, "Invalid flag given. Try \"-h\".\n" );
+					return 1;
+				}
+				run_option = GR1C_MODE_PATCH;
+				remove_goal_mode = strtol( argv[i+1], NULL, 10 );
+				if (remove_goal_mode < 0) {
+					fprintf( stderr,
+							 "System goal indices must be nonnegative.\n" );
+					return 1;
+				}
+				i++;
 			} else {
 				fprintf( stderr, "Invalid flag given. Try \"-h\".\n" );
 				return 1;
@@ -180,7 +201,8 @@ int main( int argc, char **argv )
 	}
 
 	if (edges_input_index >= 0 && clformula_index >= 0) {
-		fprintf( stderr, "\"-e\" and \"-a\" flags cannot be used simultaneously.\n" );
+		fprintf( stderr,
+				 "\"-e\" and \"-a\" flags cannot be used simultaneously.\n" );
 		return 1;
 	} else if (edges_input_index >= 0 && aut_input_index < 0) {
 		fprintf( stderr, "\"-e\" flag can only be used with \"-a\"\n" );
@@ -188,34 +210,43 @@ int main( int argc, char **argv )
 	} else if (clformula_index >= 0 && aut_input_index < 0) {
 		fprintf( stderr, "\"-f\" flag can only be used with \"-a\"\n" );
 		return 1;
+	} else if (remove_goal_mode >= 0 && aut_input_index < 0) {
+		fprintf( stderr, "\"-r\" flag can only be used with \"-a\"\n" );
+		return 1;
 	}
 
 	if (help_flag) {
 		/* Split among printf() calls to conform with ISO C90 string length */
-		printf( "Usage: %s [-hVvlp] [-m VARS] [-t TYPE] [-aeo FILE] [-f FORM] [FILE]\n\n"
+		printf( "Usage: %s [-hVvlp] [-m VARS] [-t TYPE] [-aeo FILE] [-f FORM] [-r N] [FILE]\n\n"
 				"  -h          this help message\n"
 				"  -V          print version and exit\n"
 				"  -v          be verbose; use -vv to be more verbose\n"
 				"  -l          enable logging\n"
-				"  -t TYPE     strategy output format; default is \"tulip\";\n"
-				"              supported formats: txt, dot, aut, tulip, tulip0\n"
+				"  -t TYPE     strategy output format; default is \"json\";\n"
+				"              supported formats: txt, dot, aut, json, tulip\n"
 				"  -p          dump parse trees to DOT files, and echo formulas to screen\n", argv[0] );
 		printf( "  -m VARS     VARS is a space-separated list of metric variables\n"
 				"  -a FILE     automaton input file, in gr1c \"aut\" format;\n"
 				"              if FILE is -, then read from stdin\n"
 				"  -e FILE     patch, given game edge set change file; requires -a flag\n"
-				"  -o FILE     output strategy to FILE, rather than stdout (default)\n"
-				"  -f FORM     FORM is a Boolean (state) formula, currently only\n"
-				"              used for appending a system goal; requires -a flag.\n" );
+				"  -o FILE     output strategy to FILE, rather than stdout (default)\n" );
+		printf( "  -f FORM     FORM is a Boolean (state) formula, currently only\n"
+				"              used for appending a system goal; requires -a flag.\n"
+				"  -r N        remove system goal N (in order, according to given file);\n"
+				"              requires -a flag.\n" );
 		return 1;
 	}
 
-	if (input_index < 0 && (run_option == GR1C_MODE_PATCH && !strncmp( argv[aut_input_index], "-", 1 ))) {
-		printf( "Reading spec from stdin in some cases while patching is not yet implemented.\n" );
+	if (input_index < 0 && (run_option == GR1C_MODE_PATCH
+							&& !strncmp( argv[aut_input_index], "-", 1 ))) {
+		printf( "Reading spec from stdin in some cases while patching is"
+				" not yet implemented.\n" );
 		return 1;
 	}
 	if (run_option == GR1C_MODE_UNSET) {
-		fprintf( stderr, "%s requires a patching request. Try \"-h\".\n", argv[0] );
+		fprintf( stderr,
+				 "%s requires a patching request. Try \"-h\".\n",
+				 argv[0] );
 		return 1;
 	}
 
@@ -240,7 +271,8 @@ int main( int argc, char **argv )
 
 	if (clformula_index >= 0) {
 		if (verbose > 1) {
-			logprint( "Parsing command-line formula \"%s\"...", argv[clformula_index] );
+			logprint( "Parsing command-line formula \"%s\"...",
+					  argv[clformula_index] );
 		}
 
 		clf_file = tmpfile();
@@ -265,7 +297,7 @@ int main( int argc, char **argv )
 		if (verbose > 1) {
 			logprint_startline();
 			logprint_raw( "Command-line formula, printed from ptree: " );
-			print_formula( clformula, getlogstream() );
+			print_formula( clformula, getlogstream(), FORMULA_SYNTAX_GR1C );
 			logprint_endline();
 		}
 	}
@@ -294,23 +326,64 @@ int main( int argc, char **argv )
 	if (verbose)
 		logprint( "Done." );
 
+	if (check_gr1c_form( evar_list, svar_list, env_init, sys_init,
+						 env_trans_array, et_array_len,
+						 sys_trans_array, st_array_len,
+						 env_goals, num_egoals, sys_goals, num_sgoals,
+						 UNDEFINED_INIT ) < 0)
+		return -1;
+
 	/* Close input file, if opened. */
 	if (input_index > 0)
 		fclose( fp );
 
-	/* Treat deterministic problem in which ETRANS or EINIT is omitted. */
-	if (evar_list == NULL) {
-		if (et_array_len == 0) {
-			et_array_len = 1;
-			env_trans_array = malloc( sizeof(ptree_t *) );
-			if (env_trans_array == NULL) {
-				perror( "gr1c, malloc" );
-				return -1;
-			}
-			*env_trans_array = init_ptree( PT_CONSTANT, NULL, 1 );
+	/* Check clformula for clandestine variables */
+	tmppt = NULL;
+	if (svar_list == NULL) {
+		svar_list = evar_list;
+	} else {
+		tmppt = get_list_item( svar_list, -1 );
+		tmppt->left = evar_list;
+	}
+	if ((tmpstr = check_vars( clformula, svar_list, NULL )) != NULL) {
+		fprintf( stderr, "Unrecognized variable in -f formula: %s\n", tmpstr );
+		return -1;
+	}
+	if (tmppt != NULL) {
+		tmppt->left = NULL;
+		tmppt = NULL;
+	} else {
+		svar_list = NULL;
+	}
+
+
+	/* Omission implies empty. */
+	if (et_array_len == 0) {
+		et_array_len = 1;
+		env_trans_array = malloc( sizeof(ptree_t *) );
+		if (env_trans_array == NULL) {
+			perror( "gr1c, malloc" );
+			return -1;
 		}
-		if (env_init == NULL)
-			env_init = init_ptree( PT_CONSTANT, NULL, 1 );
+		*env_trans_array = init_ptree( PT_CONSTANT, NULL, 1 );
+	}
+	if (st_array_len == 0) {
+		st_array_len = 1;
+		sys_trans_array = malloc( sizeof(ptree_t *) );
+		if (sys_trans_array == NULL) {
+			perror( "gr1c, malloc" );
+			return -1;
+		}
+		*sys_trans_array = init_ptree( PT_CONSTANT, NULL, 1 );
+	}
+	if (num_sgoals == 0) {
+		num_sgoals = 1;
+		sys_goals = malloc( sizeof(ptree_t *) );
+		if (sys_goals == NULL) {
+			perror( "gr1c, malloc" );
+			return -1;
+		}
+		*sys_goals = init_ptree( PT_CONSTANT, NULL, 1 );
 	}
 
 	/* Number of variables, before expansion of those that are nonboolean */
@@ -363,9 +436,11 @@ int main( int argc, char **argv )
 					}
 				} else {
 					if (tmppt->left == NULL) {
-						printf( "%s (%d; {0..%d})", tmppt->name, var_index, tmppt->value );
+						printf( "%s (%d; {0..%d})",
+								tmppt->name, var_index, tmppt->value );
 					} else {
-						printf( "%s (%d; {0..%d}), ", tmppt->name, var_index, tmppt->value );
+						printf( "%s (%d; {0..%d}), ",
+								tmppt->name, var_index, tmppt->value );
 					}
 				}
 				tmppt = tmppt->left;
@@ -388,9 +463,11 @@ int main( int argc, char **argv )
 					}
 				} else {
 					if (tmppt->left == NULL) {
-						printf( "%s (%d; {0..%d})", tmppt->name, var_index, tmppt->value );
+						printf( "%s (%d; {0..%d})",
+								tmppt->name, var_index, tmppt->value );
 					} else {
-						printf( "%s (%d; {0..%d}), ", tmppt->name, var_index, tmppt->value );
+						printf( "%s (%d; {0..%d}), ",
+								tmppt->name, var_index, tmppt->value );
 					}
 				}
 				tmppt = tmppt->left;
@@ -409,15 +486,17 @@ int main( int argc, char **argv )
 							&env_trans_array, &et_array_len,
 							&sys_trans_array, &st_array_len,
 							&env_goals, num_egoals, &sys_goals, num_sgoals,
-							verbose ) < 0)
+							ALL_ENV_EXIST_SYS_INIT, verbose ) < 0)
 		return -1;
-	nonbool_var_list = expand_nonbool_variables( &evar_list, &svar_list, verbose );
+	nonbool_var_list = expand_nonbool_variables( &evar_list, &svar_list,
+												 verbose );
 
 	tmppt = nonbool_var_list;
 	while (tmppt) {
 		if (clformula != NULL) {
 			if (verbose > 1)
-				logprint( "Expanding nonbool variable %s in command-line formula...", tmppt->name );
+				logprint( "Expanding nonbool variable %s in command-line"
+						  " formula...", tmppt->name );
 			clformula = expand_to_bool( clformula, tmppt->name, tmppt->value );
 			if (verbose > 1)
 				logprint( "Done." );
@@ -469,7 +548,9 @@ int main( int argc, char **argv )
 
 		if (verbose)
 			logprint( "Patching given strategy..." );
-		strategy = patch_localfixpoint( manager, strategy_fp, fp, original_num_env, original_num_sys, nonbool_var_list, offw, verbose );
+		strategy = patch_localfixpoint( manager, strategy_fp, fp,
+										original_num_env, original_num_sys,
+										nonbool_var_list, offw, verbose );
 		if (verbose)
 			logprint( "Done." );
 
@@ -481,7 +562,9 @@ int main( int argc, char **argv )
 		} else if (et_array_len == 1) {
 			env_trans = *env_trans_array;
 		} else {
-			fprintf( stderr, "Syntax error: GR(1) specification is missing environment transition rules.\n" );
+			fprintf( stderr,
+					 "Syntax error: GR(1) specification is missing environment"
+					 " transition rules.\n" );
 			return -1;
 		}
 		if (st_array_len > 1) {
@@ -489,13 +572,57 @@ int main( int argc, char **argv )
 		} else if (st_array_len == 1) {
 			sys_trans = *sys_trans_array;
 		} else {
-			fprintf( stderr, "Syntax error: GR(1) specification is missing system transition rules.\n" );
+			fprintf( stderr,
+					 "Syntax error: GR(1) specification is missing system"
+					 " transition rules.\n" );
 			return -1;
 		}
 
 		if (verbose)
 			logprint( "Patching given strategy..." );
-		strategy = add_metric_sysgoal( manager, strategy_fp, original_num_env, original_num_sys, offw, num_metric_vars, clformula, verbose );
+		strategy = add_metric_sysgoal( manager, strategy_fp,
+									   original_num_env, original_num_sys,
+									   offw, num_metric_vars, clformula,
+									   verbose );
+		if (verbose)
+			logprint( "Done." );
+
+	} else if (remove_goal_mode >= 0) {  /* rm_sysgoal() */
+
+		if (remove_goal_mode > num_sgoals-1) {
+			fprintf( stderr,
+					 "Requested system goal index %d exceeds maximum %d.\n",
+					 remove_goal_mode,
+					 num_sgoals-1 );
+			return 1;
+		}
+
+		if (et_array_len > 1) {
+			env_trans = merge_ptrees( env_trans_array, et_array_len, PT_AND );
+		} else if (et_array_len == 1) {
+			env_trans = *env_trans_array;
+		} else {
+			fprintf( stderr,
+					 "Syntax error: GR(1) specification is missing environment"
+					 " transition rules.\n" );
+			return -1;
+		}
+		if (st_array_len > 1) {
+			sys_trans = merge_ptrees( sys_trans_array, st_array_len, PT_AND );
+		} else if (st_array_len == 1) {
+			sys_trans = *sys_trans_array;
+		} else {
+			fprintf( stderr,
+					 "Syntax error: GR(1) specification is missing system"
+					 " transition rules.\n" );
+			return -1;
+		}
+
+		if (verbose)
+			logprint( "Patching given strategy..." );
+		strategy = rm_sysgoal( manager, strategy_fp,
+							   original_num_env, original_num_sys,
+							   remove_goal_mode, verbose );
 		if (verbose)
 			logprint( "Done." );
 
@@ -515,7 +642,8 @@ int main( int argc, char **argv )
 	if (strategy != NULL) {  /* De-expand nonboolean variables */
 		tmppt = nonbool_var_list;
 		while (tmppt) {
-			aut_compact_nonbool( strategy, evar_list, svar_list, tmppt->name, tmppt->value );
+			aut_compact_nonbool( strategy, evar_list, svar_list,
+								 tmppt->name, tmppt->value );
 			tmppt = tmppt->left;
 		}
 
@@ -550,8 +678,8 @@ int main( int argc, char **argv )
 			}
 		} else if (format_option == OUTPUT_FORMAT_AUT) {
 			aut_aut_dump( strategy, num_env+num_sys, fp );
-		} else if (format_option == OUTPUT_FORMAT_TULIP0) {
-			tulip0_aut_dump( strategy, evar_list, svar_list, fp );
+		} else if (format_option == OUTPUT_FORMAT_JSON) {
+			json_aut_dump( strategy, evar_list, svar_list, fp );
 		} else { /* OUTPUT_FORMAT_TULIP */
 			tulip_aut_dump( strategy, evar_list, svar_list, fp );
 		}
@@ -584,7 +712,7 @@ int main( int argc, char **argv )
 		Cudd_RecursiveDeref( manager, T );
 	if (strategy)
 		delete_aut( strategy );
-	if (verbose)
+	if (verbose > 1)
 		logprint( "Cudd_CheckZeroRef -> %d", Cudd_CheckZeroRef( manager ) );
 	Cudd_Quit(manager);
 	if (logging_flag)
