@@ -2,7 +2,7 @@
  * \brief Project-wide definitions and macros.
  *
  *
- * SCL; 2012, 2013.
+ * SCL; 2012-2014.
  */
 
 
@@ -10,8 +10,11 @@
 #define COMMON_H
 
 
-#define GR1C_VERSION "0.6.2"
-#define GR1C_COPYRIGHT "Copyright (c) 2012, 2013 by Scott C. Livingston,\nCalifornia Institute of Technology\n\nThis is free, open source software, released under a BSD license\nand without warranty."
+#define GR1C_VERSION "0.9.0"
+#define GR1C_COPYRIGHT "Copyright (c) 2012-2014 by Scott C. Livingston,\n" \
+	"California Institute of Technology\n\n" \
+	"This is free, open source software, released under a BSD license\n" \
+	"and without warranty."
 
 
 #define GR1C_INTERACTIVE_PROMPT ">>> "
