@@ -1,7 +1,7 @@
 /* ptree.c -- Definitions for signatures appearing in ptree.h.
  *
  *
- * SCL; 2012, 2013.
+ * SCL; 2012-2014.
  */
 
 
@@ -46,6 +46,28 @@ ptree_t *init_ptree( int type, char *name, int value )
 }
 
 
+ptree_t *copy_ptree( ptree_t *head )
+{
+	ptree_t *new_head;
+	if (head == NULL)
+		return NULL;
+
+	new_head = init_ptree( head->type, head->name, head->value );
+	if (new_head == NULL)
+		return NULL;
+
+	new_head->left = copy_ptree( head->left );
+	if (new_head->left == NULL && head->left != NULL)
+		return NULL;
+
+	new_head->right = copy_ptree( head->right );
+	if (new_head->right == NULL && head->right != NULL)
+		return NULL;
+
+	return new_head;
+}
+
+
 void delete_tree( ptree_t *head )
 {
 	if (head == NULL)
@@ -137,11 +159,13 @@ int rmin_tree_value( ptree_t *head, char *name )
 		return -9999;
 
 	if (head->type == PT_EQUALS) {
-		if ((head->left->type == PT_VARIABLE || head->left->type == PT_NEXT_VARIABLE)
+		if ((head->left->type == PT_VARIABLE
+			 || head->left->type == PT_NEXT_VARIABLE)
 			&& head->right->type == PT_CONSTANT
 			&& !strncmp(head->left->name, name, strlen(name))) {
 			minval = head->right->value;
-		} else if ((head->right->type == PT_VARIABLE || head->right->type == PT_NEXT_VARIABLE)
+		} else if ((head->right->type == PT_VARIABLE
+					|| head->right->type == PT_NEXT_VARIABLE)
 				   && head->left->type == PT_CONSTANT
 				   && !strncmp(head->right->name, name, strlen(name))) {
 			minval = head->left->value;
@@ -179,11 +203,13 @@ int rmax_tree_value( ptree_t *head, char *name )
 		return -9999;
 
 	if (head->type == PT_EQUALS) {
-		if ((head->left->type == PT_VARIABLE || head->left->type == PT_NEXT_VARIABLE)
+		if ((head->left->type == PT_VARIABLE
+			 || head->left->type == PT_NEXT_VARIABLE)
 			&& head->right->type == PT_CONSTANT
 			&& !strncmp(head->left->name, name, strlen(name))) {
 			maxval = head->right->value;
-		} else if ((head->right->type == PT_VARIABLE || head->right->type == PT_NEXT_VARIABLE)
+		} else if ((head->right->type == PT_VARIABLE
+					|| head->right->type == PT_NEXT_VARIABLE)
 				   && head->left->type == PT_CONSTANT
 				   && !strncmp(head->right->name, name, strlen(name))) {
 			maxval = head->left->value;
@@ -261,7 +287,7 @@ ptree_t *var_to_bool( char *name, int maxval )
 	char varname[VARNAME_STRING_LEN];
 	int i;
 
-	if (name == NULL || maxval < 1)
+	if (name == NULL || maxval < 0)
 		return NULL;
 
 	maxval = (int)(ceil(log2( maxval+1 )));
@@ -288,7 +314,18 @@ ptree_t *expand_nonbool_varnum( ptree_t *head, char *name, int maxval )
 	int i;
 
 	/* Handle pointless calls */
-	if (head == NULL || !((head->type == PT_LT) || (head->type == PT_GT) || (head->type == PT_LE) || (head->type == PT_GE) || (head->type == PT_NOTEQ)) || !(((head->left->type == PT_VARIABLE || head->left->type == PT_NEXT_VARIABLE) && head->right->type == PT_CONSTANT && !strcmp( head->left->name, name )) || ((head->right->type == PT_VARIABLE || head->right->type == PT_NEXT_VARIABLE) && head->left->type == PT_CONSTANT && !strcmp( head->right->name, name ))))
+	if (head == NULL
+		|| !((head->type == PT_LT) || (head->type == PT_GT)
+			 || (head->type == PT_LE) || (head->type == PT_GE)
+			 || (head->type == PT_NOTEQ))
+		|| !(((head->left->type == PT_VARIABLE
+			   || head->left->type == PT_NEXT_VARIABLE)
+			  && head->right->type == PT_CONSTANT
+			  && !strcmp( head->left->name, name ))
+			 || ((head->right->type == PT_VARIABLE
+				  || head->right->type == PT_NEXT_VARIABLE)
+				 && head->left->type == PT_CONSTANT
+				 && !strcmp( head->right->name, name ))))
 		return head;
 
 	op_type = head->type;
@@ -312,18 +349,22 @@ ptree_t *expand_nonbool_varnum( ptree_t *head, char *name, int maxval )
 		return init_ptree( PT_CONSTANT, NULL, 1 );  /* constant True */
 
 	if (op_type == PT_NOTEQ) {
-		heads = malloc( (maxval-1)*sizeof(ptree_t *) );
+		heads = malloc( maxval*sizeof(ptree_t *) );
 		if (heads == NULL) {
 			perror( "expand_nonbool_varnum, malloc" );
 			return NULL;
 		}
+		min = 0;
 		for (i = 0; i <= maxval; i++) {
-			if (i == this_val)
+			if (i == this_val) {
+				min = 1;
 				continue;
+			}
 			*(heads+i-min) = init_ptree( PT_EQUALS, NULL, 0 );
 			(*(heads+i-min))->left = init_ptree( var_tense, name, 0 );
 			(*(heads+i-min))->right = init_ptree( PT_CONSTANT, NULL, i );
 		}
+		head = merge_ptrees( heads, maxval, PT_OR );
 	} else {
 		if (op_type == PT_LT) {
 			min = 0;
@@ -349,8 +390,8 @@ ptree_t *expand_nonbool_varnum( ptree_t *head, char *name, int maxval )
 			(*(heads+i-min))->left = init_ptree( var_tense, name, 0 );
 			(*(heads+i-min))->right = init_ptree( PT_CONSTANT, NULL, i );
 		}
+		head = merge_ptrees( heads, max-min+1, PT_OR );
 	}
-	head = merge_ptrees( heads, max-min+1, PT_OR );
 	free( heads );
 	return head;
 }
@@ -361,13 +402,18 @@ ptree_t *expand_to_bool( ptree_t *head, char *name, int maxval )
 	ptree_t **heads;
 	int this_val, i;
 	bool is_next;
-	int num_bits = (int)(ceil(log2( maxval+1 )));
+	int num_bits;
 	ptree_t *expanded_varlist;
-	int maxval_padded = (1 << num_bits) - 1;
 
 	if (head == NULL)
 		return NULL;
 
+	if (maxval > 0) {
+		num_bits = (int)(ceil(log2( maxval+1 )));
+	} else {
+		num_bits = 1;
+	}
+
 	expanded_varlist = var_to_bool( name, maxval );
 	if (expanded_varlist == NULL)
 		return NULL;
@@ -377,7 +423,11 @@ ptree_t *expand_to_bool( ptree_t *head, char *name, int maxval )
 		|| head->type == PT_NOTEQ)
 		head = expand_nonbool_varnum( head, name, maxval );
 
-	if (head->type == PT_EQUALS && ((head->left->type != PT_CONSTANT && !strcmp( head->left->name, name )) || (head->right->type != PT_CONSTANT && !strcmp( head->right->name, name )))) {
+	if (head->type == PT_EQUALS
+		&& ((head->left->type != PT_CONSTANT
+			 && !strcmp( head->left->name, name ))
+			|| (head->right->type != PT_CONSTANT
+				&& !strcmp( head->right->name, name )))) {
 		/* We assume that nonboolean comparison is only between a
 		   variable and a number; will be generalized soon. */
 		if (head->left->type == PT_CONSTANT) {
@@ -395,10 +445,6 @@ ptree_t *expand_to_bool( ptree_t *head, char *name, int maxval )
 				is_next = True;
 			}
 		}
-		if (this_val > maxval_padded) {
-			fprintf( stderr, "Error expand_to_bool: %d is outside range of variable %s [0,%d]\n(expands to [0,%d] after including unused values).\n", this_val, name, maxval, maxval_padded );
-			exit( -1 );  /* Fatal error */
-		}
 
 		delete_tree( head );
 		heads = malloc( num_bits*sizeof(ptree_t *) );
@@ -407,6 +453,10 @@ ptree_t *expand_to_bool( ptree_t *head, char *name, int maxval )
 			return NULL;
 		}
 
+		/* Enforce inability to reach values outside the expanded domain */
+		if (this_val > (int)(pow( 2, num_bits ) )-1 || this_val < 0)
+			return init_ptree( PT_CONSTANT, NULL, 0 );
+
 		for (i = num_bits-1; i >= 0; i--) {
 			if ((this_val >> i)&1) {
 				*(heads+i) = get_list_item( expanded_varlist, i );
@@ -433,15 +483,22 @@ ptree_t *expand_to_bool( ptree_t *head, char *name, int maxval )
 }
 
 
-ptree_t *unreach_expanded_bool( char *name, int lower, int upper )
+ptree_t *unreach_expanded_bool( char *name, int lower, int upper, int type )
 {
 	ptree_t *head, *node;
 	int i;
 	if (lower > upper)
 		return NULL;
+	if (!(type == PT_VARIABLE || type == PT_NEXT_VARIABLE)) {
+		fprintf( stderr,
+				 "unreach_expanded_bool: Invoked with unsupported type, %d\n",
+				 type );
+		return NULL;
+	}
+
 	head = init_ptree( PT_NEG, NULL, 0 );
 	head->right = init_ptree( PT_EQUALS, NULL, 0 );
-	head->right->left = init_ptree( PT_NEXT_VARIABLE, name, 0 );
+	head->right->left = init_ptree( type, name, 0 );
 	head->right->right = init_ptree( PT_CONSTANT, NULL, lower );
 	for (i = lower+1; i <= upper; i++) {
 		node = head;
@@ -449,19 +506,10 @@ ptree_t *unreach_expanded_bool( char *name, int lower, int upper )
 		head->right = node;
 		head->left = init_ptree( PT_NEG, NULL, 0 );
 		head->left->right = init_ptree( PT_EQUALS, NULL, 0 );
-		head->left->right->left = init_ptree( PT_NEXT_VARIABLE, name, 0 );
-		head->left->right->right = init_ptree( PT_CONSTANT, NULL, i );
-
-		node = head;
-		head = init_ptree( PT_AND, NULL, 0 );
-		head->right = node;
-		head->left = init_ptree( PT_NEG, NULL, 0 );
-		head->left->right = init_ptree( PT_EQUALS, NULL, 0 );
-		head->left->right->left = init_ptree( PT_VARIABLE, name, 0 );
+		head->left->right->left = init_ptree( type, name, 0 );
 		head->left->right->right = init_ptree( PT_CONSTANT, NULL, i );
 	}
 
-	/* return expand_to_bool( head, name, upper ); */
 	return head;
 }
 
@@ -589,7 +637,9 @@ ptree_t *merge_ptrees( ptree_t **heads, int len, int type )
 	for (i = len-1; i > 1; i--) {
 		node->left = init_ptree( type, NULL, -1 );
 		if (node->left == NULL) {
-			fprintf( stderr, "Error: merge_ptrees failed to create enough new nodes.\n" );
+			fprintf( stderr,
+					 "Error: merge_ptrees failed to create enough new"
+					 " nodes.\n" );
 			return NULL;
 		}
 		node = node->left;
@@ -624,11 +674,15 @@ ptree_t *remove_list_item( ptree_t *head, int index )
 
 	/* Error-checking */
 	if (head == NULL) {
-		fprintf( stderr, "WARNING: remove_list_item called with empty tree.\n" );
+		fprintf( stderr,
+				 "WARNING: remove_list_item called with empty tree.\n" );
 		return NULL;
 	}
 	if (index < -1 || index >= length) {
-		fprintf( stderr, "Error: remove_list_item given invalid index, %d. Max possible is %d.\n", index, length-1 );
+		fprintf( stderr,
+				 "Error: remove_list_item given invalid index, %d."
+				 " Max possible is %d.\n",
+				 index, length-1 );
 		return NULL;
 	}
 
@@ -745,7 +799,7 @@ int tree_dot_dump( ptree_t *head, char *filename )
 }
 
 
-void print_formula( ptree_t *head, FILE *fp )
+void print_formula( ptree_t *head, FILE *fp, unsigned char format_flags )
 {
 	if (head == NULL) {
 		fprintf( stderr, "WARNING: print_formula called with NULL node." );
@@ -755,6 +809,32 @@ void print_formula( ptree_t *head, FILE *fp )
 	if (fp == NULL)
 		fp = stdout;
 
+	/* Special cases */
+	if (format_flags == FORMULA_SYNTAX_SPIN) {
+		switch (head->type) {
+		case PT_IMPLIES:
+			fprintf( fp, "(!" );
+			print_formula( head->left, fp, format_flags );
+			fprintf( fp, "||" );
+			print_formula( head->right, fp, format_flags );
+			fprintf( fp, ")" );
+			return;
+		case PT_EQUIV:
+			/* Naively compute subtrees twice, to avoid space
+			   requirements of caching. */
+			fprintf( fp, "((" );
+			print_formula( head->left, fp, format_flags );
+			fprintf( fp, "&&" );
+			print_formula( head->right, fp, format_flags );
+			fprintf( fp, ")||(!" );
+			print_formula( head->left, fp, format_flags );
+			fprintf( fp, "&&!" );
+			print_formula( head->right, fp, format_flags );
+			fprintf( fp, "))" );
+			return;
+		}
+	}
+
 	switch (head->type) {
 	case PT_AND:
 	case PT_OR:
@@ -767,12 +847,19 @@ void print_formula( ptree_t *head, FILE *fp )
 	case PT_GE:  /* ...or equal to*/
 	case PT_LE:
 		fprintf( fp, "(" );
-		print_formula( head->left, fp );
+		if (head->left != NULL && head->right != NULL
+			&& ((head->right->type == PT_VARIABLE
+				 || head->right->type == PT_NEXT_VARIABLE)
+				&& head->left->type == PT_CONSTANT && head->right->value >= 0)) {
+			fprintf( fp, "%d", head->left->value );
+		} else {
+			print_formula( head->left, fp, format_flags );
+		}
 		break;
 
 	case PT_NEG:
 		fprintf( fp, "(!" );
-		print_formula( head->right, fp );
+		print_formula( head->right, fp, format_flags );
 		fprintf( fp, ")" );
 		return;
 
@@ -781,30 +868,51 @@ void print_formula( ptree_t *head, FILE *fp )
 		return;
 
 	case PT_NEXT_VARIABLE:
-		fprintf( fp, "%s'", head->name );
+		if (format_flags == FORMULA_SYNTAX_SPIN) {
+			fprintf( fp, "%s_next", head->name );
+		} else {
+			fprintf( fp, "%s'", head->name );
+		}
 		return;
 		
 	case PT_CONSTANT:
 		if (head->value == 0) {
-			fprintf( fp, "False" );
+			if (format_flags == FORMULA_SYNTAX_SPIN) {
+				fprintf( fp, "false" );
+			} else {
+				fprintf( fp, "False" );
+			}
 		} else if (head->value == 1) {
-			fprintf( fp, "True" );
+			if (format_flags == FORMULA_SYNTAX_SPIN) {
+				fprintf( fp, "true" );
+			} else {
+				fprintf( fp, "True" );
+			}
 		} else {
 			fprintf( fp, "%d", head->value );
 		}
 		return;
 
 	default:
-		fprintf( stderr, "WARNING: print_formula called with node of unknown type" );
+		fprintf( stderr,
+				 "WARNING: print_formula called with node of unknown type" );
 		return;
 	}
 
 	switch (head->type) {
 	case PT_AND:
-		fprintf( fp, "&" );
+		if (format_flags == FORMULA_SYNTAX_SPIN) {
+			fprintf( fp, "&&" );
+		} else {
+			fprintf( fp, "&" );
+		}
 		break;
 	case PT_OR:
-		fprintf( fp, "|" );
+		if (format_flags == FORMULA_SYNTAX_SPIN) {
+			fprintf( fp, "||" );
+		} else {
+			fprintf( fp, "|" );
+		}
 		break;
 	case PT_IMPLIES:
 		fprintf( fp, "->" );
@@ -813,7 +921,11 @@ void print_formula( ptree_t *head, FILE *fp )
 		fprintf( fp, "<->" );
 		break;
 	case PT_EQUALS:
-		fprintf( fp, "=" );
+		if (format_flags == FORMULA_SYNTAX_SPIN) {
+			fprintf( fp, "==" );
+		} else {
+			fprintf( fp, "=" );
+		}
 		break;
 	case PT_NOTEQ:
 		fprintf( fp, "!=" );
@@ -831,7 +943,14 @@ void print_formula( ptree_t *head, FILE *fp )
 		fprintf( fp, "<=" );
 		break;
 	}
-	print_formula( head->right, fp );
+	if (head->left != NULL && head->right != NULL
+		&& ((head->left->type == PT_VARIABLE
+			 || head->left->type == PT_NEXT_VARIABLE)
+			&& head->right->type == PT_CONSTANT && head->left->value >= 0)) {
+		fprintf( fp, "%d", head->right->value );
+	} else {
+		print_formula( head->right, fp, format_flags );
+	}
 	fprintf( fp, ")" );
 	return;
 }
@@ -856,7 +975,10 @@ DdNode *ptree_BDD( ptree_t *head, ptree_t *var_list, DdManager *manager )
 	case PT_VARIABLE:
 		index = find_list_item( var_list, head->type, head->name, 0 );
 		if (index < 0) {
-			fprintf( stderr, "Error: ptree_BDD requested variable \"%s\", but it is not in given list.\n", head->name );
+			fprintf( stderr,
+					 "Error: ptree_BDD requested variable \"%s\","
+					 " but it is not in given list.\n",
+					 head->name );
 			exit(-1);
 		}
 		lsub = Cudd_ReadOne( manager );
@@ -870,13 +992,17 @@ DdNode *ptree_BDD( ptree_t *head, ptree_t *var_list, DdManager *manager )
 	case PT_NEXT_VARIABLE:
 		index = find_list_item( var_list, PT_VARIABLE, head->name, 0 );
 		if (index < 0) {
-			fprintf( stderr, "Error: ptree_BDD requested primed variable \"%s\", but it is not in given list.\n", head->name );
+			fprintf( stderr,
+					 "Error: ptree_BDD requested primed variable \"%s\","
+					 " but it is not in given list.\n",
+					 head->name );
 			exit(-1);
 		}
 		lsub = Cudd_ReadOne( manager );
 		Cudd_Ref( lsub );
 		fn = Cudd_bddAnd( manager, lsub,
-						  Cudd_bddIthVar( manager, tree_size(var_list)+index ) );
+						  Cudd_bddIthVar( manager,
+										  tree_size(var_list)+index ) );
 		Cudd_Ref( fn );
 		Cudd_RecursiveDeref( manager, lsub );
 		break;
@@ -959,7 +1085,8 @@ int find_list_item( ptree_t *head, int type, char *name, int value )
 		if (head->type == type) {
 			if (head->type == PT_VARIABLE || head->type == PT_NEXT_VARIABLE) {
 				/* If node is variable type, then names must match. */
-				if (name != NULL && head->name != NULL && !strcmp( head->name, name ))
+				if (name != NULL && head->name != NULL
+					&& !strcmp( head->name, name ))
 					break;
 			} else if (head->type == PT_CONSTANT) {
 				/* If node is constant (e.g., True), then values must match. */
