@@ -2,7 +2,7 @@
  *            Also see solve_operators.c
  *
  *
- * SCL; 2012, 2013.
+ * SCL; 2012-2014.
  */
 
 
@@ -29,11 +29,10 @@ extern int num_sgoals;
 
 
 DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
-								   unsigned char init_flags, unsigned char verbose );
+								   unsigned char init_flags,
+								   unsigned char verbose );
 
 
-/* N.B., we assume there is at least one system goal.  This assumption
-   will be removed in a future version (soon). */
 anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 					 unsigned char verbose )
 {
@@ -58,6 +57,7 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 
 	DdNode *tmp, *tmp2;
 	int i, j, r, k;  /* Generic counters */
+	int offset;
 	bool env_nogoal_flag = False;  /* Indicate environment has no goals */
 	int loop_mode;
 	int next_mode;
@@ -106,24 +106,36 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 	} else {
 		var_separator = get_list_item( evar_list, -1 );
 		if (var_separator == NULL) {
-			fprintf( stderr, "Error: get_list_item failed on environment variables list.\n" );
+			fprintf( stderr,
+					 "Error: get_list_item failed on environment variables"
+					 " list.\n" );
 			return NULL;
 		}
 		var_separator->left = svar_list;
 	}
 
 	/* Generate BDDs for the various parse trees from the problem spec. */
-	einit = ptree_BDD( env_init, evar_list, manager );
-	sinit = ptree_BDD( sys_init, evar_list, manager );
-	if (verbose)
+	if (env_init != NULL) {
+		einit = ptree_BDD( env_init, evar_list, manager );
+	} else {
+		einit = Cudd_ReadOne( manager );
+		Cudd_Ref( einit );
+	}
+	if (sys_init != NULL) {
+		sinit = ptree_BDD( sys_init, evar_list, manager );
+	} else {
+		sinit = Cudd_ReadOne( manager );
+		Cudd_Ref( sinit );
+	}
+	if (verbose > 1)
 		logprint( "Building environment transition BDD..." );
 	etrans = ptree_BDD( env_trans, evar_list, manager );
-	if (verbose) {
+	if (verbose > 1) {
 		logprint( "Done." );
 		logprint( "Building system transition BDD..." );
 	}
 	strans = ptree_BDD( sys_trans, evar_list, manager );
-	if (verbose)
+	if (verbose > 1)
 		logprint( "Done." );
 
 	/* Build goal BDDs, if present. */
@@ -148,9 +160,11 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 		var_separator->left = NULL;
 	}
 
-	W = compute_winning_set_BDD( manager, etrans, strans, egoals, sgoals, verbose );
+	W = compute_winning_set_BDD( manager, etrans, strans, egoals, sgoals,
+								 verbose );
 	if (W == NULL) {
-		fprintf( stderr, "Error synthesize: failed to construct winning set.\n" );
+		fprintf( stderr,
+				 "Error synthesize: failed to construct winning set.\n" );
 		return NULL;
 	}
 	Y = compute_sublevel_sets( manager, W, etrans, strans,
@@ -158,7 +172,8 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 							   sgoals, num_sgoals,
 							   &num_sublevels, &X_ijr, verbose );
 	if (Y == NULL) {
-		fprintf( stderr, "Error synthesize: failed to construct sublevel sets.\n" );
+		fprintf( stderr,
+				 "Error synthesize: failed to construct sublevel sets.\n" );
 		return NULL;
 	}
 
@@ -173,11 +188,8 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 	for (i = 0; i < num_sgoals; i++) {
 		Cudd_RecursiveDeref( manager, *(*(Y+i)) );
 		Cudd_RecursiveDeref( manager, *(*(Y+i)+1) );
-		for (r = 0; r < num_egoals; r++) {
+		for (r = 0; r < num_egoals; r++)
 			Cudd_RecursiveDeref( manager, *(*(*(X_ijr+i))+r) );
-			Cudd_RecursiveDeref( manager, *(*(*(X_ijr+i)+1)+r) );
-			*(*(*(X_ijr+i)+1)+r) = Cudd_Not( Cudd_ReadOne( manager ) );
-		}
 		free( *(*(X_ijr+i)) );
 
 		*(*(Y+i)+1) = Cudd_bddAnd( manager, *(sgoals+i), W );
@@ -189,7 +201,8 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 			*(*(X_ijr+i)+j) = *(*(X_ijr+i)+j+1);
 		}
 		*(Y+i) = realloc( *(Y+i), (*(num_sublevels+i))*sizeof(DdNode *) );
-		*(X_ijr+i) = realloc( *(X_ijr+i), (*(num_sublevels+i))*sizeof(DdNode *) );
+		*(X_ijr+i) = realloc( *(X_ijr+i),
+							  (*(num_sublevels+i))*sizeof(DdNode *) );
 		if (*(Y+i) == NULL || *(X_ijr+i) == NULL) {
 			perror( "synthesize, realloc" );
 			return NULL;
@@ -203,7 +216,9 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 	   compute_winning_set_BDD above. */
 	tmp = Cudd_bddVarMap( manager, W );
 	if (tmp == NULL) {
-		fprintf( stderr, "Error synthesize: Error in swapping variables with primed forms.\n" );
+		fprintf( stderr,
+				 "Error synthesize: Error in swapping variables with primed"
+				 " forms.\n" );
 		return NULL;
 	}
 	Cudd_Ref( tmp );
@@ -216,62 +231,191 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 	   of a state), and iterating until every reached state and mode
 	   combination has already been encountered (whence the
 	   strategy already built). */
-	if (init_flags == ALL_SYS_INIT) {
-		tmp = Cudd_bddAnd( manager, einit, sinit );
+	if (init_flags == ALL_INIT
+		|| (init_flags == ONE_SIDE_INIT && sys_init == NULL)) {
+		if (init_flags == ALL_INIT) {
+			if (verbose > 1)
+				logprint( "Enumerating initial states, given init_flags ="
+						  " ALL_INIT" );
+			tmp = Cudd_bddAnd( manager, einit, sinit );
+		} else {
+			if (verbose > 1)
+				logprint( "Enumerating initial states, given init_flags ="
+						  " ONE_SIDE_INIT and empty SYSINIT" );
+			tmp = einit;
+			Cudd_Ref( tmp );
+		}
 		Cudd_Ref( tmp );
 		Cudd_AutodynDisable( manager );
 		Cudd_ForeachCube( manager, tmp, gen, gcube, gvalue ) {
 			initialize_cube( state, gcube, num_env+num_sys );
 			while (!saturated_cube( state, gcube, num_env+num_sys )) {
-				this_node_stack = insert_anode( this_node_stack, 0, -1, state, num_env+num_sys );
+				this_node_stack = insert_anode( this_node_stack, 0, -1, False,
+												state, num_env+num_sys );
 				if (this_node_stack == NULL) {
-					fprintf( stderr, "Error synthesize: building list of initial states.\n" );
+					fprintf( stderr,
+							 "Error synthesize: building list of initial"
+							 " states.\n" );
 					return NULL;
 				}
 				increment_cube( state, gcube, num_env+num_sys );
 			}
-			this_node_stack = insert_anode( this_node_stack, 0, -1, state, num_env+num_sys );
+			this_node_stack = insert_anode( this_node_stack, 0, -1, False,
+											state, num_env+num_sys );
 			if (this_node_stack == NULL) {
-				fprintf( stderr, "Error synthesize: building list of initial states.\n" );
+				fprintf( stderr,
+						 "Error synthesize: building list of initial"
+						 " states.\n" );
 				return NULL;
 			}
 		}
 		Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
 		Cudd_RecursiveDeref( manager, tmp );
-	} else { /* EXIST_SYS_INIT */
-		tmp = Cudd_bddAnd( manager, einit, sinit );
-		Cudd_Ref( tmp );
-		tmp2 = Cudd_bddAnd( manager, tmp, W );
+
+	} else if (init_flags == ALL_ENV_EXIST_SYS_INIT) {
+		if (verbose > 1)
+			logprint( "Enumerating initial states, given init_flags ="
+					  " ALL_ENV_EXIST_SYS_INIT" );
+
+		/* Generate initial environment states */
+		for (i = 0; i < num_env; i++)
+			*(cube+i) = 2;
+		for (i = num_env; i < 2*(num_sys+num_env); i++)
+			*(cube+i) = 1;
+		tmp2 = Cudd_CubeArrayToBdd( manager, cube );
+		if (tmp2 == NULL) {
+			fprintf( stderr, "Error in generating cube for quantification.\n" );
+			return NULL;
+		}
 		Cudd_Ref( tmp2 );
-		Cudd_RecursiveDeref( manager, tmp );
+		tmp = Cudd_bddExistAbstract( manager, einit, tmp2 );
+		if (tmp == NULL) {
+			fprintf( stderr, "Error in performing quantification.\n" );
+			return NULL;
+		}
+		Cudd_Ref( tmp );
+		Cudd_RecursiveDeref( manager, tmp2 );
+
 		Cudd_AutodynDisable( manager );
-		Cudd_ForeachCube( manager, tmp2, gen, gcube, gvalue ) {
-			initialize_cube( state, gcube, num_env+num_sys );
-			while (!saturated_cube( state, gcube, num_env+num_sys )) {
-				this_node_stack = insert_anode( this_node_stack, 0, -1, state, num_env+num_sys );
+		Cudd_ForeachCube( manager, tmp, gen, gcube, gvalue ) {
+			initialize_cube( state, gcube, num_env );
+			while (!saturated_cube( state, gcube, num_env )) {
+				this_node_stack = insert_anode( this_node_stack, 0, -1, False,
+												state, num_env+num_sys );
 				if (this_node_stack == NULL) {
-					fprintf( stderr, "Error synthesize: building list of initial states.\n" );
+					fprintf( stderr,
+							 "Error synthesize: building list of initial"
+							 " states.\n" );
 					return NULL;
 				}
-				increment_cube( state, gcube, num_env+num_sys );
+				increment_cube( state, gcube, num_env );
 			}
-			this_node_stack = insert_anode( this_node_stack, 0, -1, state, num_env+num_sys );
+			this_node_stack = insert_anode( this_node_stack, 0, -1, False,
+											state, num_env+num_sys );
 			if (this_node_stack == NULL) {
-				fprintf( stderr, "Error synthesize: building list of initial states.\n" );
+				fprintf( stderr,
+						 "Error synthesize: building list of initial"
+						 " states.\n" );
 				return NULL;
 			}
 		}
 		Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
-		Cudd_RecursiveDeref( manager, tmp2 );
+		Cudd_RecursiveDeref( manager, tmp );
+
+		/* For each initial environment state, find a system state in
+		   the winning set W. */
+		node = this_node_stack;
+		while (node) {
+			for (i = num_env; i < 2*(num_env+num_sys); i++)
+				*(cube+i) = 2;
+			for (i = 0; i < num_env; i++)
+				*(cube+i) = *(node->state+i);
+
+			tmp2 = Cudd_CubeArrayToBdd( manager, cube );
+			if (tmp2 == NULL) {
+				fprintf( stderr, "Error in generating cube for cofactor.\n" );
+				return NULL;
+			}
+			Cudd_Ref( tmp2 );
+
+			tmp = Cudd_Cofactor( manager, W, tmp2 );
+			if (tmp == NULL) {
+				fprintf( stderr, "Error in computing cofactor.\n" );
+				return NULL;
+			}
+			Cudd_Ref( tmp );
+			Cudd_RecursiveDeref( manager, tmp2 );
+
+			tmp2 = Cudd_bddAnd( manager, tmp, sinit );
+			Cudd_Ref( tmp2 );
+			Cudd_RecursiveDeref( manager, tmp );
+
+			Cudd_AutodynDisable( manager );
+			gen = Cudd_FirstCube( manager, tmp2, &gcube, &gvalue );
+			if (gen == NULL) {
+				fprintf( stderr, "Error synthesize: failed to find cube.\n" );
+				return NULL;
+			}
+			if (Cudd_IsGenEmpty( gen )) {
+				fprintf( stderr,
+						 "Error synthesize: unexpected losing initial"
+						 " environment state found.\n" );
+				return NULL;
+			}
+			initialize_cube( state, gcube, num_env+num_sys );
+			for (i = num_env; i < num_env+num_sys; i++)
+				*(node->state+i) = *(state+i);
+			Cudd_GenFree( gen );
+			Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
+			Cudd_RecursiveDeref( manager, tmp2 );
+
+			node = node->next;
+		}
+
+	} else if (init_flags == ONE_SIDE_INIT) {
+		/* N.B., case of sys_init==NULL is treated above */
+		if (verbose > 1)
+			logprint( "Enumerating initial states, given init_flags ="
+					  " ONE_SIDE_INIT and empty ENVINIT" );
+
+		tmp = Cudd_bddAnd( manager, sinit, W );
+		Cudd_Ref( tmp );
+		Cudd_AutodynDisable( manager );
+		gen = Cudd_FirstCube( manager, tmp, &gcube, &gvalue );
+		if (gen == NULL) {
+			fprintf( stderr, "Error synthesize: failed to find cube.\n" );
+			return NULL;
+		}
+		if (Cudd_IsGenEmpty( gen )) {
+			fprintf( stderr,
+					 "Error synthesize: no winning initial state found.\n" );
+			return NULL;
+		}
+		initialize_cube( state, gcube, num_env+num_sys );
+		this_node_stack = insert_anode( this_node_stack, 0, -1, False,
+										state, num_env+num_sys );
+		if (this_node_stack == NULL) {
+			fprintf( stderr,
+					 "Error synthesize: building list of initial states.\n" );
+			return NULL;
+		}
+		Cudd_GenFree( gen );
+		Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
+		Cudd_RecursiveDeref( manager, tmp );
+	} else {
+		fprintf( stderr, "Error: Unrecognized init_flags %d", init_flags );
+		return NULL;
 	}
 
 	/* Insert all stacked, initial nodes into strategy. */
 	node = this_node_stack;
 	while (node) {
-		strategy = insert_anode( strategy, node->mode, node->rgrad,
+		strategy = insert_anode( strategy, node->mode, node->rgrad, True,
 								 node->state, num_env+num_sys );
 		if (strategy == NULL) {
-			fprintf( stderr, "Error synthesize: inserting state node into strategy.\n" );
+			fprintf( stderr,
+					 "Error synthesize: inserting state node into"
+					 " strategy.\n" );
 			return NULL;
 		}
 		node = node->next;
@@ -287,7 +431,8 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 			j = *(num_sublevels+this_node_stack->mode);
 			do {
 				j--;
-				ddval = Cudd_Eval( manager, *(*(Y+this_node_stack->mode)+j), cube );
+				ddval = Cudd_Eval( manager,
+								   *(*(Y+this_node_stack->mode)+j), cube );
 				if (ddval->type.value < .1) {
 					j++;
 					break;
@@ -304,8 +449,8 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 			}
 		} while (loop_mode != this_node_stack->mode);
 		if (this_node_stack->mode == loop_mode) {
-			node = find_anode( strategy, this_node_stack->mode, this_node_stack->state,
-							   num_env+num_sys );
+			node = find_anode( strategy, this_node_stack->mode,
+							   this_node_stack->state, num_env+num_sys );
 			if (node->trans_len > 0) {
 				/* This state and mode combination is already in strategy. */
 				this_node_stack = pop_anode( this_node_stack );
@@ -321,17 +466,25 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 				continue;
 			} else {
 				strategy = delete_anode( strategy, node );
-				new_node = find_anode( strategy, this_node_stack->mode, this_node_stack->state, num_env+num_sys );
+				new_node = find_anode( strategy, this_node_stack->mode,
+									   this_node_stack->state,
+									   num_env+num_sys );
 				if (new_node == NULL) {
-					strategy = insert_anode( strategy, this_node_stack->mode, -1, this_node_stack->state, num_env+num_sys );
+					strategy = insert_anode( strategy,
+											 this_node_stack->mode, -1, False,
+											 this_node_stack->state,
+											 num_env+num_sys );
 					if (strategy == NULL) {
-						fprintf( stderr, "Error synthesize: inserting state node into strategy.\n" );
+						fprintf( stderr,
+								 "Error synthesize: inserting state node into"
+								 " strategy.\n" );
 						return NULL;
 					}
-					new_node = find_anode( strategy, this_node_stack->mode, this_node_stack->state, num_env+num_sys );
+					new_node = find_anode( strategy, this_node_stack->mode,
+										   this_node_stack->state,
+										   num_env+num_sys );
 				} else if (new_node->trans_len > 0) {
 					replace_anode_trans( strategy, node, new_node );
-					/* This state and mode combination is already in strategy. */
 					this_node_stack = pop_anode( this_node_stack );
 					continue;
 				}
@@ -361,7 +514,9 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 				Y_i_primed = Cudd_bddVarMap( manager, *(*(Y+node->mode)+j-1) );
 			}
 			if (Y_i_primed == NULL) {
-				fprintf( stderr, "Error synthesize: Error in swapping variables with primed forms.\n" );
+				fprintf( stderr,
+						 "Error synthesize: Error in swapping variables with"
+						 " primed forms.\n" );
 				return NULL;
 			}
 			Cudd_Ref( Y_i_primed );
@@ -393,12 +548,17 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 				Cudd_GenFree( gen );
 				Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
 				if (j > 0) {
+					for (offset = 1; offset >= 0; offset--) {
 					for (r = 0; r < num_egoals; r++) {
 						Cudd_RecursiveDeref( manager, tmp );
 						Cudd_RecursiveDeref( manager, Y_i_primed );
-						Y_i_primed = Cudd_bddVarMap( manager, *(*(*(X_ijr+node->mode)+j)+r) );
+						Y_i_primed
+							= Cudd_bddVarMap( manager,
+											  *(*(*(X_ijr+node->mode)+j - offset)+r) );
 						if (Y_i_primed == NULL) {
-							fprintf( stderr, "Error synthesize: Error in swapping variables with primed forms.\n" );
+							fprintf( stderr,
+									 "Error synthesize: Error in swapping"
+									 " variables with primed forms.\n" );
 							return NULL;
 						}
 						Cudd_Ref( Y_i_primed );
@@ -417,11 +577,20 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 							tmp = tmp2;
 						}
 					
-						if (!(Cudd_bddLeq( manager, tmp, Cudd_Not( Cudd_ReadOne( manager ) ) )*Cudd_bddLeq( manager, Cudd_Not( Cudd_ReadOne( manager ) ), tmp )))
+						if (!(Cudd_bddLeq( manager, tmp,
+										   Cudd_Not( Cudd_ReadOne( manager ) ) )
+							  *Cudd_bddLeq( manager,
+											Cudd_Not( Cudd_ReadOne( manager ) ),
+											tmp )))
 							break;
 					}
+					if (r < num_egoals)
+						break;
+					}
 					if (r >= num_egoals) {
-						fprintf( stderr, "Error synthesize: unexpected losing state.\n" );
+						fprintf( stderr,
+								 "Error synthesize: unexpected losing"
+								 " state.\n" );
 						return NULL;
 					}
 				} else {
@@ -449,12 +618,14 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 				Cudd_AutodynDisable( manager );
 				gen = Cudd_FirstCube( manager, tmp, &gcube, &gvalue );
 				if (gen == NULL) {
-					fprintf( stderr, "Error synthesize: failed to find cube.\n" );
+					fprintf( stderr,
+							 "Error synthesize: failed to find cube.\n" );
 					return NULL;
 				}
 				if (Cudd_IsGenEmpty( gen )) {
 					Cudd_GenFree( gen );
-					fprintf( stderr, "Error synthesize: unexpected losing state.\n" );
+					fprintf( stderr,
+							 "Error synthesize: unexpected losing state.\n" );
 					return NULL;
 				}
 				for (i = 0; i < 2*(num_env+num_sys); i++)
@@ -485,18 +656,24 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 				}
 			}
 
-			new_node = find_anode( strategy, next_mode, state, num_env+num_sys );
+			new_node = find_anode( strategy, next_mode,
+								   state, num_env+num_sys );
 			if (new_node == NULL) {
-				strategy = insert_anode( strategy, next_mode, -1,
+				strategy = insert_anode( strategy, next_mode, -1, False,
 										 state, num_env+num_sys );
 				if (strategy == NULL) {
-					fprintf( stderr, "Error synthesize: inserting new node into strategy.\n" );
+					fprintf( stderr,
+							 "Error synthesize: inserting new node into"
+							 " strategy.\n" );
 					return NULL;
 				}
 				this_node_stack = insert_anode( this_node_stack, next_mode, -1,
+												False,
 												state, num_env+num_sys );
 				if (this_node_stack == NULL) {
-					fprintf( stderr, "Error synthesize: pushing node onto stack failed.\n" );
+					fprintf( stderr,
+							 "Error synthesize: pushing node onto stack"
+							 " failed.\n" );
 					return NULL;
 				}
 			} 
@@ -506,7 +683,9 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 										   num_env+num_sys,
 										   next_mode, state );
 			if (strategy == NULL) {
-				fprintf( stderr, "Error synthesize: inserting new transition into strategy.\n" );
+				fprintf( stderr,
+						 "Error synthesize: inserting new transition into"
+						 " strategy.\n" );
 				return NULL;
 			}
 
@@ -538,11 +717,6 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 		free( sgoals );
 	free( cube );
 	free( state );
-	if (env_nogoal_flag) {
-		num_egoals = 0;
-		delete_tree( *env_goals );
-		free( env_goals );
-	}
 	for (i = 0; i < num_sgoals; i++) {
 		for (j = 0; j < *(num_sublevels+i); j++) {
 			Cudd_RecursiveDeref( manager, *(*(Y+i)+j) );
@@ -561,6 +735,12 @@ anode_t *synthesize( DdManager *manager,  unsigned char init_flags,
 		free( X_ijr );
 		free( num_sublevels );
 	}
+	if (env_nogoal_flag) {
+		num_egoals = 0;
+		delete_tree( *env_goals );
+		free( env_goals );
+	}
+
 
 	return strategy;
 }
@@ -575,11 +755,12 @@ DdNode *check_realizable( DdManager *manager, unsigned char init_flags,
 
 
 DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
-								   unsigned char init_flags, unsigned char verbose )
+								   unsigned char init_flags,
+								   unsigned char verbose )
 {
 	bool realizable;
-	DdNode *tmp, *tmp2;
-	DdNode *einit, *sinit;
+	DdNode *tmp, *tmp2, *tmp3;
+	DdNode *einit = NULL, *sinit = NULL;
 
 	ptree_t *var_separator;
 	int num_env, num_sys;
@@ -587,6 +768,13 @@ DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
 				   account for both variables and their primes). */
 	DdNode *ddcube;
 
+	if (verbose > 1) {
+		logprint_startline();
+		logprint_raw( "check_realizable_internal invoked with init_flags: " );
+		LOGPRINT_INIT_FLAGS( init_flags );
+		logprint_endline();
+	}
+
 	num_env = tree_size( evar_list );
 	num_sys = tree_size( svar_list );
 
@@ -606,14 +794,26 @@ DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
 	} else {
 		var_separator = get_list_item( evar_list, -1 );
 		if (var_separator == NULL) {
-			fprintf( stderr, "Error: get_list_item failed on environment variables list.\n" );
+			fprintf( stderr,
+					 "Error: get_list_item failed on environment variables"
+					 " list.\n" );
 			return NULL;
 		}
 		var_separator->left = svar_list;
 	}
 
-	einit = ptree_BDD( env_init, evar_list, manager );
-	sinit = ptree_BDD( sys_init, evar_list, manager );
+	if (env_init != NULL) {
+		einit = ptree_BDD( env_init, evar_list, manager );
+	} else {
+		einit = Cudd_ReadOne( manager );
+		Cudd_Ref( einit );
+	}
+	if (sys_init != NULL) {
+		sinit = ptree_BDD( sys_init, evar_list, manager );
+	} else {
+		sinit = Cudd_ReadOne( manager );
+		Cudd_Ref( sinit );
+	}
 
 	/* Break the link that appended the system variables list to the
 	   environment variables list. */
@@ -623,22 +823,34 @@ DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
 		var_separator->left = NULL;
 	}
 
-	/* Does winning set contain all initial states? */
-	if (init_flags == ALL_SYS_INIT) {
+	/* Does winning set contain all initial states?
+
+	   We assume that the initial condition formulae, i.e., env_init
+	   and sys_init, are appropriate for the given init_flags.  This
+	   can be checked with check_gr1c_form() (cf. gr1c_util.h). */
+	if (init_flags == ALL_INIT) {
+
 		tmp = Cudd_bddAnd( manager, einit, sinit );
 		Cudd_Ref( tmp );
 		tmp2 = Cudd_bddAnd( manager, tmp, W );
 		Cudd_Ref( tmp2 );
-		if (!(Cudd_bddLeq( manager, tmp, tmp2 )*Cudd_bddLeq( manager, tmp2, tmp ))) {
+		if (!(Cudd_bddLeq( manager, tmp, tmp2 )
+			  *Cudd_bddLeq( manager, tmp2, tmp ))) {
 			realizable = False;
 		} else {
 			realizable = True;
 		}
 		Cudd_RecursiveDeref( manager, tmp );
 		Cudd_RecursiveDeref( manager, tmp2 );
-	} else { /* EXIST_SYS_INIT */
-		tmp = Cudd_bddAnd( manager, sinit, W );
+
+	} else if (init_flags == ALL_ENV_EXIST_SYS_INIT) {
+
+		tmp = Cudd_bddAnd( manager, sinit, einit );
 		Cudd_Ref( tmp );
+
+		tmp3 = Cudd_bddAnd( manager, tmp, W );
+		Cudd_Ref( tmp3 );
+
 		cube_sys( cube, num_env, num_sys );
 		ddcube = Cudd_CubeArrayToBdd( manager, cube );
 		if (ddcube == NULL) {
@@ -652,34 +864,59 @@ DdNode *check_realizable_internal( DdManager *manager, DdNode *W,
 			return NULL;
 		}
 		Cudd_Ref( tmp2 );
-		Cudd_RecursiveDeref( manager, ddcube );
 		Cudd_RecursiveDeref( manager, tmp );
 
-		tmp = Cudd_bddOr( manager, Cudd_Not( einit ), tmp2 );
-		Cudd_Ref( tmp );
-		Cudd_RecursiveDeref( manager, tmp2 );
-		cube_env( cube, num_env, num_sys );
-		ddcube = Cudd_CubeArrayToBdd( manager, cube );
-		if (ddcube == NULL) {
-			fprintf( stderr, "Error in generating cube for quantification." );
-			return NULL;
-		}
-		Cudd_Ref( ddcube );
-		tmp2 = Cudd_bddUnivAbstract( manager, tmp, ddcube );
-		if (tmp2 == NULL) {
+		tmp = Cudd_bddExistAbstract( manager, tmp3, ddcube );
+		if (tmp == NULL) {
 			fprintf( stderr, "Error in performing quantification." );
 			return NULL;
 		}
-		Cudd_Ref( tmp2 );
+		Cudd_Ref( tmp );
 		Cudd_RecursiveDeref( manager, ddcube );
-		Cudd_RecursiveDeref( manager, tmp );
+		Cudd_RecursiveDeref( manager, tmp3 );
 
-		if (!(Cudd_bddLeq( manager, tmp2, Cudd_ReadOne( manager ) )*Cudd_bddLeq( manager, Cudd_ReadOne( manager ), tmp2 ))) {
+		if (!(Cudd_bddLeq( manager, tmp, tmp2 )
+			  *Cudd_bddLeq( manager, tmp2, tmp ))) {
 			realizable = False;
 		} else {
 			realizable = True;
 		}
+		Cudd_RecursiveDeref( manager, tmp );
 		Cudd_RecursiveDeref( manager, tmp2 );
+
+	} else if (init_flags == ONE_SIDE_INIT) {
+		if (sys_init == NULL) {
+
+			tmp = Cudd_bddAnd( manager, einit, W );
+			Cudd_Ref( tmp );
+			if (!(Cudd_bddLeq( manager, tmp, einit )
+				  *Cudd_bddLeq( manager, einit, tmp ))) {
+				realizable = False;
+			} else {
+				realizable = True;
+			}
+			Cudd_RecursiveDeref( manager, tmp );
+
+		} else {
+			/* If env_init and sys_init were both NULL, we would still
+			   want to treat env_init as True.  Cf. ONE_SIDE_INIT
+			   option for init_flags in documentation for
+			   check_realizable(). */
+
+			tmp = Cudd_bddAnd( manager, sinit, W );
+			Cudd_Ref( tmp );
+			if (!(Cudd_bddLeq( manager, tmp, Cudd_Not( Cudd_ReadOne( manager ) ) )
+				  *Cudd_bddLeq( manager, Cudd_Not( Cudd_ReadOne( manager ) ), tmp ))) {
+				realizable = True;
+			} else {
+				realizable = False;
+			}
+			Cudd_RecursiveDeref( manager, tmp );
+		}
+
+	} else {
+		fprintf( stderr, "Error: Unrecognized init_flags %d", init_flags );
+		return NULL;
 	}
 
 	Cudd_RecursiveDeref( manager, einit );
