@@ -1,7 +1,7 @@
 /* util.c -- Small handy routines; declarations appear in gr1c_util.h
  *
  *
- * SCL; 2012, 2013.
+ * SCL; 2012-2014.
  */
 
 
@@ -12,6 +12,9 @@
 #include <string.h>
 
 #include "common.h"
+#include "ptree.h"
+#include "solve.h"
+#include "solve_support.h"
 #include "gr1c_util.h"
 #include "logging.h"
 
@@ -64,18 +67,22 @@ ptree_t *expand_nonbool_variables( ptree_t **evar_list, ptree_t **svar_list,
 	} else {
 		var_separator = get_list_item( (*evar_list), -1 );
 		if (var_separator == NULL) {
-			fprintf( stderr, "Error: get_list_item failed on environment variables list.\n" );
+			fprintf( stderr,
+					 "Error: get_list_item failed on environment variables"
+					 " list.\n" );
 			return NULL;
 		}
 		var_separator->left = (*svar_list);
 	}
 	tmppt = (*evar_list);
 	while (tmppt) {
-		if (tmppt->value > 0) {
+		if (tmppt->value >= 0) {
 			if (nonbool_var_list == NULL) {
-				nonbool_var_list = init_ptree( PT_VARIABLE, tmppt->name, tmppt->value );
+				nonbool_var_list = init_ptree( PT_VARIABLE,
+											   tmppt->name, tmppt->value );
 			} else {
-				append_list_item( nonbool_var_list, PT_VARIABLE, tmppt->name, tmppt->value );
+				append_list_item( nonbool_var_list, PT_VARIABLE,
+								  tmppt->name, tmppt->value );
 			}
 
 		}
@@ -90,15 +97,17 @@ ptree_t *expand_nonbool_variables( ptree_t **evar_list, ptree_t **svar_list,
 	/* Expand the variable list */
 	if ((*evar_list) != NULL) {
 		tmppt = (*evar_list);
-		if (tmppt->value > 0) {  /* Handle special case of head node */
+		if (tmppt->value >= 0) {  /* Handle special case of head node */
 			expt = var_to_bool( tmppt->name, tmppt->value );
 			(*evar_list) = expt;
 			prevpt = get_list_item( expt, -1 );
 			prevpt->left = tmppt->left;
+		} else {
+			prevpt = tmppt;
 		}
 		tmppt = tmppt->left;
 		while (tmppt) {
-			if (tmppt->value > 0) {
+			if (tmppt->value >= 0) {
 				expt = var_to_bool( tmppt->name, tmppt->value );
 				prevpt->left = expt;
 				prevpt = get_list_item( expt, -1 );
@@ -112,15 +121,17 @@ ptree_t *expand_nonbool_variables( ptree_t **evar_list, ptree_t **svar_list,
 
 	if ((*svar_list) != NULL) {
 		tmppt = (*svar_list);
-		if (tmppt->value > 0) {  /* Handle special case of head node */
+		if (tmppt->value >= 0) {  /* Handle special case of head node */
 			expt = var_to_bool( tmppt->name, tmppt->value );
 			(*svar_list) = expt;
 			prevpt = get_list_item( expt, -1 );
 			prevpt->left = tmppt->left;
+		} else {
+			prevpt = tmppt;
 		}
 		tmppt = tmppt->left;
 		while (tmppt) {
-			if (tmppt->value > 0) {
+			if (tmppt->value >= 0) {
 				expt = var_to_bool( tmppt->name, tmppt->value );
 				prevpt->left = expt;
 				prevpt = get_list_item( expt, -1 );
@@ -136,7 +147,8 @@ ptree_t *expand_nonbool_variables( ptree_t **evar_list, ptree_t **svar_list,
 }
 
 
-vartype *expand_nonbool_state( vartype *state, int *offw, int num_nonbool, int mapped_len )
+vartype *expand_nonbool_state( vartype *state, int *offw, int num_nonbool,
+							   int mapped_len )
 {
 	int i, j, k;
 	vartype *mapped_state, *state_frag;
@@ -177,43 +189,114 @@ int expand_nonbool_GR1( ptree_t *evar_list, ptree_t *svar_list,
 						ptree_t ***sys_trans_array, int *st_array_len,
 						ptree_t ***env_goals, int num_env_goals,
 						ptree_t ***sys_goals, int num_sys_goals,
+						unsigned char init_flags,
 						unsigned char verbose )
 {
 	int i;
 	ptree_t *tmppt, *prevpt, *var_separator;
 	int maxbitval;
 
-	/* Handle "don't care" bits */
+	/* Make nonzero settings of "don't care" bits unreachable */
 	tmppt = evar_list;
 	while (tmppt) {
-		maxbitval = (int)(pow( 2, ceil(log2( tmppt->value+1 )) ));
+		if (tmppt->value == -1) {  /* Ignore Boolean variables */
+			tmppt = tmppt->left;
+			continue;
+		}
+
+		maxbitval = (int)(pow( 2, ceil(log2( tmppt->value > 0 ?
+											 tmppt->value+1 : 2 )) ));
 		if (maxbitval-1 > tmppt->value) {
 			if (verbose > 1)
-				logprint( "In mapping %s to a bitvector, blocking values %d-%d", tmppt->name, tmppt->value+1, maxbitval-1 );
-			(*et_array_len)++;
-			(*env_trans_array) = realloc( (*env_trans_array), sizeof(ptree_t *)*(*et_array_len) );
+				logprint( "In mapping %s to a bitvector, blocking values %d-%d",
+						  tmppt->name, tmppt->value+1, maxbitval-1 );
+
+			/* Initial conditions */
+			if (init_flags == ONE_SIDE_INIT && *sys_init != NULL) {
+				prevpt = *sys_init;
+				*sys_init = init_ptree( PT_AND, NULL, 0 );
+				(*sys_init)->right
+					= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+											 maxbitval-1, PT_VARIABLE );
+				(*sys_init)->left = prevpt;
+			} else {
+				if (*env_init == NULL)
+					*env_init = init_ptree( PT_CONSTANT, NULL, 1 );
+				prevpt = *env_init;
+				*env_init = init_ptree( PT_AND, NULL, 0 );
+				(*env_init)->right
+					= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+											 maxbitval-1, PT_VARIABLE );
+				(*env_init)->left = prevpt;
+			}
+
+			/* Transition rules */
+			(*et_array_len) += 2;
+			(*env_trans_array) = realloc( (*env_trans_array),
+										  sizeof(ptree_t *)*(*et_array_len) );
 			if ((*env_trans_array) == NULL ) {
 				perror( "expand_nonbool_GR1, realloc" );
 				return -1;
 			}
-			*((*env_trans_array)+(*et_array_len)-1) = unreach_expanded_bool( tmppt->name, tmppt->value+1, maxbitval-1 );
+			*((*env_trans_array)+(*et_array_len)-2)
+				= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+										 maxbitval-1, PT_VARIABLE );
+			*((*env_trans_array)+(*et_array_len)-1)
+				= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+										 maxbitval-1, PT_NEXT_VARIABLE );
 		}
 		tmppt = tmppt->left;
 	}
 
 	tmppt = svar_list;
 	while (tmppt) {
-		maxbitval = (int)(pow( 2, ceil(log2( tmppt->value+1 )) ));
+		if (tmppt->value == -1) {  /* Ignore Boolean variables */
+			tmppt = tmppt->left;
+			continue;
+		}
+
+		maxbitval = (int)(pow( 2, ceil(log2( tmppt->value > 0 ?
+											 tmppt->value+1 : 2 )) ));
 		if (maxbitval-1 > tmppt->value) {
 			if (verbose > 1)
-				logprint( "In mapping %s to a bitvector, blocking values %d-%d", tmppt->name, tmppt->value+1, maxbitval-1 );
-			(*st_array_len)++;
-			(*sys_trans_array) = realloc( (*sys_trans_array), sizeof(ptree_t *)*(*st_array_len) );
+				logprint( "In mapping %s to a bitvector, blocking values %d-%d",
+						  tmppt->name, tmppt->value+1, maxbitval-1 );
+
+			/* Initial conditions */
+			if (init_flags == ONE_SIDE_INIT && *sys_init == NULL) {
+				if (*env_init == NULL)
+					*env_init = init_ptree( PT_CONSTANT, NULL, 1 );
+				prevpt = *env_init;
+				*env_init = init_ptree( PT_AND, NULL, 0 );
+				(*env_init)->right
+					= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+											 maxbitval-1, PT_VARIABLE );
+				(*env_init)->left = prevpt;
+			} else {
+				if (*sys_init == NULL)
+					*sys_init = init_ptree( PT_CONSTANT, NULL, 1 );
+				prevpt = *sys_init;
+				*sys_init = init_ptree( PT_AND, NULL, 0 );
+				(*sys_init)->right
+					= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+											 maxbitval-1, PT_VARIABLE );
+				(*sys_init)->left = prevpt;
+			}
+
+			/* Transition rules */
+			(*st_array_len) += 2;
+			(*sys_trans_array) = realloc( (*sys_trans_array),
+										  sizeof(ptree_t *)*(*st_array_len) );
 			if ((*sys_trans_array) == NULL ) {
 				perror( "expand_nonbool_GR1, realloc" );
 				return -1;
 			}
-			*((*sys_trans_array)+(*st_array_len)-1) = unreach_expanded_bool( tmppt->name, tmppt->value+1, maxbitval-1 );
+			*((*sys_trans_array)+(*st_array_len)-2)
+				= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+										 maxbitval-1, PT_VARIABLE );
+			*((*sys_trans_array)+(*st_array_len)-1)
+				= unreach_expanded_bool( tmppt->name, tmppt->value+1,
+										 maxbitval-1, PT_NEXT_VARIABLE );
 		}
 		tmppt = tmppt->left;
 	}
@@ -225,34 +308,57 @@ int expand_nonbool_GR1( ptree_t *evar_list, ptree_t *svar_list,
 	} else {
 		var_separator = get_list_item( evar_list, -1 );
 		if (var_separator == NULL) {
-			fprintf( stderr, "Error: get_list_item failed on environment variables list.\n" );
+			fprintf( stderr,
+					 "Error: get_list_item failed on environment variables"
+					 " list.\n" );
 			return -1;
 		}
 		var_separator->left = svar_list;
 	}
 	tmppt = evar_list;
 	while (tmppt) {
-		if (tmppt->value > 0) {
-			if (verbose > 1)
-				logprint( "Expanding nonbool variable %s in SYSINIT...", tmppt->name );
-			(*sys_init) = expand_to_bool( (*sys_init), tmppt->name, tmppt->value );
-			if (verbose > 1) {
-				logprint( "Done." );
-				logprint( "Expanding nonbool variable %s in ENVINIT...", tmppt->name );
+		if (tmppt->value >= 0) {
+			if (*sys_init != NULL) {
+				if (verbose > 1)
+					logprint( "Expanding nonbool variable %s in SYSINIT...",
+							  tmppt->name );
+				(*sys_init) = expand_to_bool( (*sys_init),
+											  tmppt->name, tmppt->value );
+				if ((*sys_init) == NULL) {
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean in SYSINIT.\n" );
+					return -1;
+				}
+				if (verbose > 1)
+					logprint( "Done." );
 			}
-			(*env_init) = expand_to_bool( (*env_init), tmppt->name, tmppt->value );
-			if (verbose > 1)
-				logprint( "Done." );
-			if ((*sys_init) == NULL || (*env_init) == NULL) {
-				fprintf( stderr, "Error expand_nonbool_GR1: Failed to convert non-Boolean variable to Boolean.\n" );
-				return -1;
+			if (*env_init != NULL) {
+				if (verbose > 1)
+					logprint( "Expanding nonbool variable %s in ENVINIT...",
+							  tmppt->name );
+				(*env_init) = expand_to_bool( (*env_init),
+											  tmppt->name, tmppt->value );
+				if ((*env_init) == NULL) {
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean in ENVINIT.\n" );
+					return -1;
+				}
+				if (verbose > 1)
+					logprint( "Done." );
 			}
 			for (i = 0; i < *et_array_len; i++) {
 				if (verbose > 1)
-					logprint( "Expanding nonbool variable %s in ENVTRANS %d...", tmppt->name, i );
-				*((*env_trans_array)+i) = expand_to_bool( *((*env_trans_array)+i), tmppt->name, tmppt->value );
+					logprint( "Expanding nonbool variable %s in ENVTRANS %d...",
+							  tmppt->name, i );
+				*((*env_trans_array)+i)
+					= expand_to_bool( *((*env_trans_array)+i),
+									  tmppt->name, tmppt->value );
 				if (*((*env_trans_array)+i) == NULL) {
-					fprintf( stderr, "Error expand_nonbool_GR1: Failed to convert non-Boolean variable to Boolean.\n" );
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean.\n" );
 					return -1;
 				}
 				if (verbose > 1)
@@ -260,10 +366,15 @@ int expand_nonbool_GR1( ptree_t *evar_list, ptree_t *svar_list,
 			}
 			for (i = 0; i < *st_array_len; i++) {
 				if (verbose > 1)
-					logprint( "Expanding nonbool variable %s in SYSTRANS %d...", tmppt->name, i );
-				*((*sys_trans_array)+i) = expand_to_bool( *((*sys_trans_array)+i), tmppt->name, tmppt->value );
+					logprint( "Expanding nonbool variable %s in SYSTRANS %d...",
+							  tmppt->name, i );
+				*((*sys_trans_array)+i)
+					= expand_to_bool( *((*sys_trans_array)+i),
+									  tmppt->name, tmppt->value );
 				if (*((*sys_trans_array)+i) == NULL) {
-					fprintf( stderr, "Error expand_nonbool_GR1: Failed to convert non-Boolean variable to Boolean.\n" );
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean.\n" );
 					return -1;
 				}
 				if (verbose > 1)
@@ -271,10 +382,15 @@ int expand_nonbool_GR1( ptree_t *evar_list, ptree_t *svar_list,
 			}
 			for (i = 0; i < num_env_goals; i++) {
 				if (verbose > 1)
-					logprint( "Expanding nonbool variable %s in ENVGOAL %d...", tmppt->name, i );
-				*((*env_goals)+i) = expand_to_bool( *((*env_goals)+i), tmppt->name, tmppt->value );
+					logprint( "Expanding nonbool variable %s in ENVGOAL %d...",
+							  tmppt->name, i );
+				*((*env_goals)+i)
+					= expand_to_bool( *((*env_goals)+i),
+									  tmppt->name, tmppt->value );
 				if (*((*env_goals)+i) == NULL) {
-					fprintf( stderr, "Error expand_nonbool_GR1: Failed to convert non-Boolean variable to Boolean.\n" );
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean.\n" );
 					return -1;
 				}
 				if (verbose > 1)
@@ -282,10 +398,15 @@ int expand_nonbool_GR1( ptree_t *evar_list, ptree_t *svar_list,
 			}
 			for (i = 0; i < num_sys_goals; i++) {
 				if (verbose > 1)
-					logprint( "Expanding nonbool variable %s in SYSGOAL %d...", tmppt->name, i );
-				*((*sys_goals)+i) = expand_to_bool( *((*sys_goals)+i), tmppt->name, tmppt->value );
+					logprint( "Expanding nonbool variable %s in SYSGOAL %d...",
+							  tmppt->name, i );
+				*((*sys_goals)+i)
+					= expand_to_bool( *((*sys_goals)+i),
+									  tmppt->name, tmppt->value );
 				if (*((*sys_goals)+i) == NULL) {
-					fprintf( stderr, "Error expand_nonbool_GR1: Failed to convert non-Boolean variable to Boolean.\n" );
+					fprintf( stderr,
+							 "Error expand_nonbool_GR1: Failed to convert"
+							 " non-Boolean variable to Boolean.\n" );
 					return -1;
 				}
 				if (verbose > 1)
@@ -342,11 +463,11 @@ void print_GR1_spec( ptree_t *evar_list, ptree_t *svar_list,
 	logprint_raw( ";" ); logprint_endline();
 
 	logprint_startline(); logprint_raw( "ENV INIT:  " );
-	print_formula( env_init, getlogstream() );
+	print_formula( env_init, getlogstream(), FORMULA_SYNTAX_GR1C );
 	logprint_raw( ";" ); logprint_endline();
 
 	logprint_startline(); logprint_raw( "SYS INIT:  " );
-	print_formula( sys_init, getlogstream() );
+	print_formula( sys_init, getlogstream(), FORMULA_SYNTAX_GR1C );
 	logprint_raw( ";" ); logprint_endline();
 
 	logprint_startline(); logprint_raw( "ENV TRANS:  " );
@@ -354,10 +475,11 @@ void print_GR1_spec( ptree_t *evar_list, ptree_t *svar_list,
 		logprint_raw( "(none)" );
 	} else {
 		logprint_raw( "[] " );
-		print_formula( *env_trans_array, getlogstream() );
+		print_formula( *env_trans_array, getlogstream(), FORMULA_SYNTAX_GR1C );
 		for (i = 1; i < et_array_len; i++) {
 			logprint_raw( " & [] " );
-			print_formula( *(env_trans_array+i), getlogstream() );
+			print_formula( *(env_trans_array+i), getlogstream(),
+						   FORMULA_SYNTAX_GR1C);
 		}
 	}
 	logprint_raw( ";" ); logprint_endline();
@@ -367,10 +489,11 @@ void print_GR1_spec( ptree_t *evar_list, ptree_t *svar_list,
 		logprint_raw( "(none)" );
 	} else {
 		logprint_raw( "[] " );
-		print_formula( *sys_trans_array, getlogstream() );
+		print_formula( *sys_trans_array, getlogstream(), FORMULA_SYNTAX_GR1C );
 		for (i = 1; i < st_array_len; i++) {
 			logprint_raw( " & [] " );
-			print_formula( *(sys_trans_array+i), getlogstream() );
+			print_formula( *(sys_trans_array+i), getlogstream(),
+						   FORMULA_SYNTAX_GR1C );
 		}
 	}
 	logprint_raw( ";" ); logprint_endline();
@@ -380,10 +503,11 @@ void print_GR1_spec( ptree_t *evar_list, ptree_t *svar_list,
 		logprint_raw( "(none)" );
 	} else {
 		logprint_raw( "[]<> " );
-		print_formula( *env_goals, getlogstream() );
+		print_formula( *env_goals, getlogstream(), FORMULA_SYNTAX_GR1C );
 		for (i = 1; i < num_env_goals; i++) {
 			logprint_raw( " & []<> " );
-			print_formula( *(env_goals+i), getlogstream() );
+			print_formula( *(env_goals+i), getlogstream(),
+						   FORMULA_SYNTAX_GR1C );
 		}
 	}
 	logprint_raw( ";" ); logprint_endline();
@@ -393,10 +517,11 @@ void print_GR1_spec( ptree_t *evar_list, ptree_t *svar_list,
 		logprint_raw( "(none)" );
 	} else {
 		logprint_raw( "[]<> " );
-		print_formula( *sys_goals, getlogstream() );
+		print_formula( *sys_goals, getlogstream(), FORMULA_SYNTAX_GR1C );
 		for (i = 1; i < num_sys_goals; i++) {
 			logprint_raw( " & []<> " );
-			print_formula( *(sys_goals+i), getlogstream() );
+			print_formula( *(sys_goals+i), getlogstream(),
+						   FORMULA_SYNTAX_GR1C );
 		}
 	}
 	logprint_raw( ";" ); logprint_endline();
@@ -413,28 +538,52 @@ int check_gr1c_form( ptree_t *evar_list, ptree_t *svar_list,
 					 ptree_t **env_trans_array, int et_array_len,
 					 ptree_t **sys_trans_array, int st_array_len,
 					 ptree_t **env_goals, int num_env_goals,
-					 ptree_t **sys_goals, int num_sys_goals )
+					 ptree_t **sys_goals, int num_sys_goals,
+					 unsigned char init_flags )
 {
 	ptree_t *tmppt;
 	char *tmpstr;
 	int i;
 
-	if (evar_list != NULL) {
-		if (env_init == NULL) {
-			fprintf( stderr, "Syntax error: GR(1) specification is missing ENVINIT.\n" );
-			return -1;
-		} else if (et_array_len == 0) {
-			fprintf( stderr, "Syntax error: GR(1) specification is missing ENVTRANS.\n" );
+	if (tree_size( evar_list ) == 0 && tree_size( svar_list ) == 0) {
+		fprintf( stderr, "Error: No variables declared.\n" );
+		return -1;
+	}
+
+	if (init_flags == ALL_ENV_EXIST_SYS_INIT) {
+		if (env_init != NULL) {
+			if ((tmpstr = check_vars( env_init, evar_list, NULL )) != NULL) {
+				fprintf( stderr,
+						 "Error: ENVINIT in GR(1) spec contains"
+						 " unexpected variable: %s,\ngiven interpretation"
+						 " from init_flags = ALL_ENV_EXIST_SYS_INIT.\n",
+						 tmpstr );
+				free( tmpstr );
+				return -1;
+			}
+		}
+
+		if (sys_init != NULL) {
+			if ((tmpstr = check_vars( sys_init, svar_list, NULL )) != NULL) {
+				fprintf( stderr,
+						 "Error: SYSINIT in GR(1) spec contains"
+						 " unexpected variable: %s,\ngiven interpretation"
+						 " from init_flags = ALL_ENV_EXIST_SYS_INIT.\n",
+						 tmpstr );
+				free( tmpstr );
+				return -1;
+			}
+		}
+		
+	} else if (init_flags == ONE_SIDE_INIT) {
+		if (env_init != NULL && sys_init != NULL) {
+			fprintf( stderr,
+					 "Syntax error: init_flags = ONE_SIDE_INIT and"
+					 " GR(1) specification has\nboth ENV_INIT and"
+					 " SYS_INIT nonempty.\n" );
 			return -1;
 		}
 	}
-	if (sys_init == NULL) {
-		fprintf( stderr, "Syntax error: GR(1) specification is missing SYSINIT.\n" );
-		return -1;
-	} else if (st_array_len == 0) {
-		fprintf( stderr, "Syntax error: GR(1) specification is missing SYSTRANS.\n" );
-		return -1;
-	}
 
 	tmppt = NULL;
 	if (svar_list == NULL) {
@@ -443,39 +592,61 @@ int check_gr1c_form( ptree_t *evar_list, ptree_t *svar_list,
 		tmppt = get_list_item( svar_list, -1 );
 		tmppt->left = evar_list;
 	}
-	if ((tmpstr = check_vars( env_init, svar_list, NULL )) != NULL) {
-		fprintf( stderr, "Error: ENVINIT in GR(1) spec contains unexpected variable: %s\n", tmpstr );
+	if (env_init != NULL
+		&& (tmpstr = check_vars( env_init, svar_list, NULL )) != NULL) {
+		fprintf( stderr,
+				 "Error: ENVINIT in GR(1) spec contains unexpected variable:"
+				 " %s\n", tmpstr );
 		free( tmpstr );
 		return -1;
-	} else if ((tmpstr = check_vars( sys_init, svar_list, NULL )) != NULL) {
-		fprintf( stderr, "Error: SYSINIT in GR(1) spec contains unexpected variable: %s\n", tmpstr );
+	} else if (sys_init != NULL
+			   && (tmpstr = check_vars( sys_init, svar_list, NULL )) != NULL) {
+		fprintf( stderr,
+				 "Error: SYSINIT in GR(1) spec contains unexpected variable:"
+				 " %s\n", tmpstr );
 		free( tmpstr );
 		return -1;
 	}
 	for (i = 0; i < et_array_len; i++) {
-		if ((tmpstr = check_vars( *(env_trans_array+i), svar_list, evar_list )) != NULL) {
-			fprintf( stderr, "Error: part %d of ENVTRANS in GR(1) spec contains unexpected variable: %s\n", i+1, tmpstr );
+		if ((tmpstr = check_vars( *(env_trans_array+i),
+								  svar_list, evar_list )) != NULL) {
+			fprintf( stderr,
+					 "Error: part %d of ENVTRANS in GR(1) spec contains"
+					 " unexpected variable: %s\n",
+					 i+1, tmpstr );
 			free( tmpstr );
 			return -1;
 		}
 	}
 	for (i = 0; i < st_array_len; i++) {
-		if ((tmpstr = check_vars( *(sys_trans_array+i), svar_list, svar_list )) != NULL) {
-			fprintf( stderr, "Error: part %d of SYSTRANS in GR(1) spec contains unexpected variable: %s\n", i+1, tmpstr );
+		if ((tmpstr = check_vars( *(sys_trans_array+i),
+								  svar_list, svar_list )) != NULL) {
+			fprintf( stderr,
+					 "Error: part %d of SYSTRANS in GR(1) spec contains"
+					 " unexpected variable: %s\n",
+					 i+1, tmpstr );
 			free( tmpstr );
 			return -1;
 		}
 	}
 	for (i = 0; i < num_env_goals; i++) {
-		if ((tmpstr = check_vars( *(env_goals+i), svar_list, NULL )) != NULL) {
-			fprintf( stderr, "Error: part %d of ENVGOAL in GR(1) spec contains unexpected variable: %s\n", i+1, tmpstr );
+		if ((tmpstr = check_vars( *(env_goals+i),
+								  svar_list, NULL )) != NULL) {
+			fprintf( stderr,
+					 "Error: part %d of ENVGOAL in GR(1) spec contains"
+					 " unexpected variable: %s\n",
+					 i+1, tmpstr );
 			free( tmpstr );
 			return -1;
 		}
 	}
 	for (i = 0; i < num_sys_goals; i++) {
-		if ((tmpstr = check_vars( *(sys_goals+i), svar_list, NULL )) != NULL) {
-			fprintf( stderr, "Error: part %d of SYSGOAL in GR(1) spec contains unexpected variable: %s\n", i+1, tmpstr );
+		if ((tmpstr = check_vars( *(sys_goals+i),
+								  svar_list, NULL )) != NULL) {
+			fprintf( stderr,
+					 "Error: part %d of SYSGOAL in GR(1) spec contains"
+					 " unexpected variable: %s\n",
+					 i+1, tmpstr );
 			free( tmpstr );
 			return -1;
 		}
@@ -493,7 +664,8 @@ int check_gr1c_form( ptree_t *evar_list, ptree_t *svar_list,
 
 /* N.B., we assume that noonbool_var_list is sorted with respect to
    the expanded variable list (evar_list followed by svar_list). */
-int *get_offsets_list( ptree_t *evar_list, ptree_t *svar_list, ptree_t *nonbool_var_list )
+int *get_offsets_list( ptree_t *evar_list, ptree_t *svar_list,
+					   ptree_t *nonbool_var_list )
 {
 	int *offw = NULL;
 	ptree_t *var, *var_tail, *tmppt;
@@ -527,7 +699,9 @@ int *get_offsets_list( ptree_t *evar_list, ptree_t *svar_list, ptree_t *nonbool_
 				start_index++;
 			}
 			if (var == NULL) {
-				fprintf( stderr, "Error get_offsets_list: Could not find match for \"%s\"\n", tmppt->name );
+				fprintf( stderr,
+						 "Error get_offsets_list: Could not find match for"
+						 " \"%s\"\n", tmppt->name );
 				free( offw );
 				return NULL;
 			}
@@ -536,7 +710,8 @@ int *get_offsets_list( ptree_t *evar_list, ptree_t *svar_list, ptree_t *nonbool_
 		var_tail = var;
 		stop_index = start_index;
 		while (var_tail->left) {
-			if (strstr( var_tail->left->name, tmppt->name ) != var_tail->left->name )
+			if (strstr( var_tail->left->name, tmppt->name )
+				!= var_tail->left->name )
 				break;
 			var_tail = var_tail->left;
 			stop_index++;
@@ -550,3 +725,59 @@ int *get_offsets_list( ptree_t *evar_list, ptree_t *svar_list, ptree_t *nonbool_
 
 	return offw;
 }
+
+
+void print_support( DdManager *manager, int state_len, DdNode *X, FILE *outf )
+{
+	FILE *prev_logf;
+	int prev_logoptions;
+	vartype *state;
+	int i;
+	DdGen *gen;
+	CUDD_VALUE_TYPE gvalue;
+	int *gcube;
+
+	if (outf != NULL) {
+		prev_logf = getlogstream();
+		prev_logoptions = getlogopt();
+		setlogstream( outf );
+		setlogopt( LOGOPT_NOTIME );
+	}
+
+	/* State vector (i.e., valuation of the variables) */
+	state = malloc( sizeof(vartype)*(state_len) );
+	if (state == NULL) {
+		perror( "print_support, malloc" );
+		return;
+	}
+
+	Cudd_AutodynDisable( manager );
+	Cudd_ForeachCube( manager, X, gen, gcube, gvalue ) {
+		initialize_cube( state, gcube, state_len );
+		while (!saturated_cube( state, gcube, state_len )) {
+			logprint_startline();
+			for (i = 0; i < state_len; i++) {
+				if (i > 0 && i % 4 == 0)
+					logprint_raw( " " );
+				logprint_raw( "%d", *(state+i) );
+			}
+			logprint_endline();
+			increment_cube( state, gcube, state_len );
+		}
+		logprint_startline();
+		for (i = 0; i < state_len; i++) {
+			if (i > 0 && i % 4 == 0)
+				logprint_raw( " " );
+			logprint_raw( "%d", *(state+i) );
+		}
+		logprint_endline();
+	}
+	Cudd_AutodynEnable( manager, CUDD_REORDER_SAME );
+
+	free( state );
+
+	if (outf != NULL) {
+		setlogstream( prev_logf );
+		setlogopt( prev_logoptions );
+	}
+}
